@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2013-2014  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2013-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -113,6 +113,7 @@ enum hwsim_tx_control_flags {
 };
 
 #define IEEE80211_TX_RATE_TABLE_SIZE	4
+#define HWSIM_DELAY_MIN_MS		1
 
 struct hwsim_rule {
 	unsigned int id;
@@ -125,6 +126,7 @@ struct hwsim_rule {
 	uint32_t frequency;
 	int priority;
 	int signal;
+	int delay;
 };
 
 struct hwsim_support {
@@ -1106,7 +1108,8 @@ static bool radio_match_addr(const struct radio_info_rec *radio,
 
 static void process_rules(const struct radio_info_rec *src_radio,
 				const struct radio_info_rec *dst_radio,
-				struct hwsim_frame *frame, bool *drop)
+				struct hwsim_frame *frame, bool *drop,
+				uint32_t *delay)
 {
 	const struct l_queue_entry *rule_entry;
 
@@ -1149,6 +1152,9 @@ static void process_rules(const struct radio_info_rec *src_radio,
 			frame->signal = rule->signal / 100;
 
 		*drop = rule->drop;
+
+		if (delay)
+			*delay = rule->delay;
 	}
 }
 
@@ -1232,7 +1238,7 @@ static void hwsim_frame_unref(struct hwsim_frame *frame)
 			bool drop = false;
 
 			process_rules(frame->ack_radio, frame->src_radio,
-					frame, &drop);
+					frame, &drop, NULL);
 
 			if (!drop)
 				frame->flags |= HWSIM_TX_STAT_ACK;
@@ -1350,10 +1356,23 @@ static bool interface_info_match_dst(const void *a, const void *b)
 		!memcmp(rec->addr, dst->addr, ETH_ALEN);
 }
 
+static void frame_delay_callback(struct l_timeout *timeout, void *user_data)
+{
+	struct send_frame_info *send_info = user_data;
+
+	if (send_frame(send_info, send_frame_callback,
+					send_frame_destroy))
+		send_info->frame->pending_callback_count++;
+	else
+		send_frame_destroy(send_info);
+
+	l_timeout_remove(timeout);
+}
+
 /*
  * Process frames in a similar way to how the kernel built-in hwsim medium
  * does this, with an additional optimization for unicast frames and
- * additonal modifications to frames decided by user-configurable rules.
+ * additional modifications to frames decided by user-configurable rules.
  */
 static void process_frame(struct hwsim_frame *frame)
 {
@@ -1361,13 +1380,14 @@ static void process_frame(struct hwsim_frame *frame)
 	bool drop_mcast = false;
 
 	if (util_is_broadcast_address(frame->dst_ether_addr))
-		process_rules(frame->src_radio, NULL, frame, &drop_mcast);
+		process_rules(frame->src_radio, NULL, frame, &drop_mcast, NULL);
 
 	for (entry = l_queue_get_entries(radio_info); entry;
 			entry = entry->next) {
 		struct radio_info_rec *radio = entry->data;
 		struct send_frame_info *send_info;
 		bool drop = drop_mcast;
+		uint32_t delay = HWSIM_DELAY_MIN_MS;
 
 		if (radio == frame->src_radio)
 			continue;
@@ -1400,7 +1420,7 @@ static void process_frame(struct hwsim_frame *frame)
 				continue;
 		}
 
-		process_rules(frame->src_radio, radio, frame, &drop);
+		process_rules(frame->src_radio, radio, frame, &drop, &delay);
 
 		if (drop)
 			continue;
@@ -1409,11 +1429,11 @@ static void process_frame(struct hwsim_frame *frame)
 		send_info->radio = radio;
 		send_info->frame = hwsim_frame_ref(frame);
 
-		if (send_frame(send_info, send_frame_callback,
-					send_frame_destroy))
-			frame->pending_callback_count++;
-		else
+		if (!l_timeout_create_ms(delay, frame_delay_callback,
+						send_info, NULL)) {
+			l_error("Error delaying frame, frame will be dropped");
 			send_frame_destroy(send_info);
+		}
 	}
 
 	hwsim_frame_unref(frame);
@@ -1862,6 +1882,7 @@ static struct l_dbus_message *rule_add(struct l_dbus *dbus,
 	rule->id = next_rule_id++;
 	rule->source_any = true;
 	rule->destination_any = true;
+	rule->delay = HWSIM_DELAY_MIN_MS;
 
 	if (!rules)
 		rules = l_queue_new();
@@ -2147,31 +2168,73 @@ static struct l_dbus_message *rule_property_set_drop(
 	return l_dbus_message_new_method_return(message);
 }
 
+static bool rule_property_get_delay(struct l_dbus *dbus,
+					struct l_dbus_message *message,
+					struct l_dbus_message_builder *builder,
+					void *user_data)
+{
+	struct hwsim_rule *rule = user_data;
+
+	l_dbus_message_builder_append_basic(builder, 'u', &rule->delay);
+
+	return true;
+}
+
+static struct l_dbus_message *rule_property_set_delay(
+					struct l_dbus *dbus,
+					struct l_dbus_message *message,
+					struct l_dbus_message_iter *new_value,
+					l_dbus_property_complete_cb_t complete,
+					void *user_data)
+{
+	struct hwsim_rule *rule = user_data;
+	uint32_t val;
+
+	if (!l_dbus_message_iter_get_variant(new_value, "u", &val) ||
+				val < HWSIM_DELAY_MIN_MS)
+		return dbus_error_invalid_args(message);
+
+	rule->delay = val;
+
+	return l_dbus_message_new_method_return(message);
+}
+
 static void setup_rule_interface(struct l_dbus_interface *interface)
 {
 	l_dbus_interface_method(interface, "Remove", 0, rule_remove, "", "");
 
-	l_dbus_interface_property(interface, "Source", 0, "s",
+	l_dbus_interface_property(interface, "Source",
+					L_DBUS_PROPERTY_FLAG_AUTO_EMIT, "s",
 					rule_property_get_source,
 					rule_property_set_source);
-	l_dbus_interface_property(interface, "Destination", 0, "s",
+	l_dbus_interface_property(interface, "Destination",
+					L_DBUS_PROPERTY_FLAG_AUTO_EMIT, "s",
 					rule_property_get_destination,
 					rule_property_set_destination);
-	l_dbus_interface_property(interface, "Bidirectional", 0, "b",
+	l_dbus_interface_property(interface, "Bidirectional",
+					L_DBUS_PROPERTY_FLAG_AUTO_EMIT, "b",
 					rule_property_get_bidirectional,
 					rule_property_set_bidirectional);
-	l_dbus_interface_property(interface, "Frequency", 0, "u",
+	l_dbus_interface_property(interface, "Frequency",
+					L_DBUS_PROPERTY_FLAG_AUTO_EMIT, "u",
 					rule_property_get_frequency,
 					rule_property_set_frequency);
-	l_dbus_interface_property(interface, "Priority", 0, "n",
+	l_dbus_interface_property(interface, "Priority",
+					L_DBUS_PROPERTY_FLAG_AUTO_EMIT, "n",
 					rule_property_get_priority,
 					rule_property_set_priority);
-	l_dbus_interface_property(interface, "SignalStrength", 0, "n",
+	l_dbus_interface_property(interface, "SignalStrength",
+					L_DBUS_PROPERTY_FLAG_AUTO_EMIT, "n",
 					rule_property_get_signal,
 					rule_property_set_signal);
-	l_dbus_interface_property(interface, "Drop", 0, "b",
+	l_dbus_interface_property(interface, "Drop",
+					L_DBUS_PROPERTY_FLAG_AUTO_EMIT, "b",
 					rule_property_get_drop,
 					rule_property_set_drop);
+	l_dbus_interface_property(interface, "Delay",
+					L_DBUS_PROPERTY_FLAG_AUTO_EMIT, "u",
+					rule_property_get_delay,
+					rule_property_set_delay);
 }
 
 static void request_name_callback(struct l_dbus *dbus, bool success,
@@ -2186,7 +2249,7 @@ static void ready_callback(void *user_data)
 	l_dbus_name_acquire(dbus, HWSIM_SERVICE, false, false, true,
 				request_name_callback, NULL);
 
-	if (!l_dbus_object_manager_enable(dbus))
+	if (!l_dbus_object_manager_enable(dbus, "/"))
 		l_info("Unable to register the ObjectManager");
 }
 
